/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.locale;

import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;

import woolpack.config.CacheMap;
import woolpack.config.ConfigExpression;

/**
 * {@link Locale}で{@link Map}をキャッシュし現在のスレッドの{@link Locale}でキャッシュを取り出す{@link CacheMap}。
 * 適用しているパターン：Proxy。
 * @author nakamura
 *
 */
public class LocaleCacheMap extends CacheMap {
	private final ThreadLocal<Locale> threadLocal;

	/**
	 * コンストラクタ。
	 * @param cacheMap キャッシュ。本クラスはこの引数の状態を変化させる。null が指定された場合はキャッシュを行わない。{@link LinkedHashMap#removeEldestEntry(java.util.Map.Entry)}を指定した{@link LinkedHashMap}を指定するとキャッシュの容量を制御することができる。{@link java.util.HashMap}などの強参照の{@link Map}を指定すると容量制限のないキャッシュになる。{@link java.util.WeakHashMap}を指定すると弱参照によるキャッシュになる。
	 * @param mapMaker {@link Map}を生成し加工するための委譲先{@link ConfigExpression}。
	 * @param threadLocal 現在のスレッドの{@link Locale}を保持する{@link ThreadLocal}。本クラスはこの引数の状態を変化させない。
	 * @throws NullPointerException 引数のいずれかが null の場合。
	 */
	public LocaleCacheMap(final Map<Object,Map<String,Object>> cacheMap, final ConfigExpression mapMaker, final ThreadLocal<Locale> threadLocal) {
		super(cacheMap, mapMaker);
		threadLocal.getClass();
		this.threadLocal = threadLocal;
	}

	/**
	 * 現在のスレッドに対して threadLocal に null でない値が設定されている場合はその値を返す。そうでない場合は継承元の返却値を返す。
	 */
	@Override public Object getCacheKey(){
		final Locale l = threadLocal.get();
		return (l!=null)?l:super.getCacheKey();
	}
}
