/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.utils;

import java.util.Map;

/**
 * {@link Map}のビルダ。
 * コンストラクタツリー内で{@link Map}の内容を生成する際に使用する。
 * @author nakamura
 *
 * @param <K> キー。
 * @param <V> 値。
 */
public class MapBuilder<K,V> {
	private final Map<K,V> map;
	private V beforeValue;
	
	/**
	 * インスタンスを返す。
	 * @param <K> キー。
	 * @param <V> 値。
	 * @param map 作用対象の{@link Map}。本クラスはこの引数の状態を変化させる。
	 * @return インスタンス。
	 * @throws NullPointerException 引数が null の場合。
	 */
	public static <K,V> MapBuilder<K,V> get(final Map<K,V> map){
		return new MapBuilder<K,V>(map);
	}
	
	private MapBuilder(final Map<K,V> map){
		map.getClass();
		this.map = map;
	}
	
	/**
	 * 作用対象の{@link Map}に対し{@link Map#put(Object, Object)}を実行する。
	 * @param key キー。
	 * @param value 値。
	 * @return このオブジェクトへの参照。
	 */
	public MapBuilder<K,V> put(final K key, final V value){
		map.put(key, value);
		beforeValue = value;
		return this;
	}

	/**
	 * 作用対象の{@link Map}に対し{@link Map#put(Object, Object)}を実行する。
	 * 前回実行した{@link #put(Object, Object)}の値を値として格納する。
	 * @param key キー。
	 * @return このオブジェクトへの参照。
	 */
	public MapBuilder<K,V> put(final K key){
		map.put(key, beforeValue);
		return this;
	}
	
	/**
	 * 作用対象の{@link Map}に対し{@link Map#putAll(java.util.Map)}を実行する。
	 * @param other 格納されるマッピング。
	 * @return このオブジェクトへの参照。
	 */
	public MapBuilder<K,V> putAll(final Map<K,V> other){
		map.putAll(other);
		return this;
	}
	
	/**
	 * 作用対象の{@link Map}を返す。
	 * @return {@link Map}。
	 */
	public Map<K,V> get(){
		return map;
	}
}
